/*
 * Copyright 2011 Federal Chancellery Austria and
 * Graz University of Technology
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.gv.util.config;

import at.gv.util.ssl.JaxWsSSLConfiguration;
import at.gv.util.xsd.szr.pvp.PvpTokenType;

/**
 * ZUSE-Util configuration interface.
 * 
 * @author <a href="mailto:Arne.Tauber@egiz.gv.at">Arne Tauber</a>
 *
 */
public interface EgovUtilConfiguration {

	JaxWsSSLConfiguration getMOASPSSsslConfiguration();
	JaxWsSSLConfiguration getMOAIDsslConfiguration();
	JaxWsSSLConfiguration getMOAZSsslConfiguration();
	JaxWsSSLConfiguration getMISsslConfiguration();
	JaxWsSSLConfiguration getMMSsslConfiguration();
	
	// SZR client config
	JaxWsSSLConfiguration getSZRsslConfiguration();
	boolean isSZRTestEnvironment();
	PvpTokenType getPVPToken();
	String getSZRProductionEnvironmentURL();
	String getSZRTestEnvironmentURL();
	
	// UR client config
	JaxWsSSLConfiguration getURsslConfiguration();
	boolean isURTestEnvironment();
	String getURProductionEnvironmentURL();
	String getURTestEnvironmentURL();
	String getURSecondaryProductionEnvironmentURL();
	PvpTokenType getURPVPToken();
	
  JaxWsSSLConfiguration getClearingsslConfiguration();
	
	// usp client config
	boolean isUSPTestEnvironment();
	String getUSPTestEnvironmentURL();
	String getUSPProductionEnvironmentURL();
	String getUSPApplicationId();
	JaxWsSSLConfiguration getUSPsslConfiguration();
	PvpTokenType getUSPPVPToken();
	int getHTTPRequestTimeout();
	
	//MISSTORK Client config
	boolean isMISSTORKEnvironment();
	String getMISSTORKProductionEnvironmentURL();
	String getMISSTORKTestEnvironmentURL();
	JaxWsSSLConfiguration getMISSTORKsslConfiguration();
	
	//ERsB Client Config
	boolean isERsBTestEnvironment();
	PvpTokenType getERsBPVPToken();
	String getERsBProductionEnvironmentURL();
	String getERsBTestEnvironmentURL();
	JaxWsSSLConfiguration getERsBsslConfiguration();
}
